<!doctype html>
<html>
<head>
<title>Flask MongoEngine</title>
<link rel=stylesheet type=text/css href="{{ url_for('static', filename='style.css') }}">
</head>
<body>
<div class=page>
  <h1>Toolbar example</h1>
  <div>
  {% block body %}{% endblock %}
  </div>
</div>
</body>
</html>
