{% extends "layout.html" %}
{% block body %}
  {% for todo in todos %}
    <div>
      <h2>{{ todo.title }}</h2>
      {{ todo.text|safe }}
    </div>
  {% else %}
    <em>Unbelievable.  No todos here so far <a href="/add">Add one</a></em>
  {% endfor %}
  <br/>
  <a href="{{ url_for('pagination') }}">See pagination</a>
{% endblock %}
